--[[
	Pathfinding for hexagonal grid.
	
	This ONLY applies to this game. This cannot be ported (easily) to another game for pathfinding.
	
	This is only to be used with the AI.
]]

local AStar={}
AStar.__index=AStar
local hexes=require 'lib/hexDataTable'

--[[
	Start: Starting hexagon
	Goal: End hexagon(where it's going)
	Grid: the grid that contains all the hexagons
]]

--[[
	f=g+h
]]
function remove(tab,val)
	--miscellaneous function for removing a given value from a table
	for index,value in pairs(tab) do
		if value==val then
			table.remove(tab,index)
			break
		end
	end
end

function find(tab,val)
	--miscellaneous function for finding if a given value is inside a table
	for index,value in pairs(tab) do
		if value==val then
			return true
		end
	end
	return false -- if the loop finished (returning will stop the execution of a function), then it never returned true.
end

function AStar.heuristic(a,b)
	-- get distance from a,b
	local function dist(a,b)
		local abs=math.abs
		return abs(a.q - b.q) + abs(a.r - b.r);
	end
	
	local d = dist(a,b)
	return d
end

function round2(num, numDecimalPlaces)
  return tonumber(string.format("%." .. (numDecimalPlaces or 0) .. "f", num))
end

function AStar.pathfind(start,goal,grid)
	local closedset={} -- unevaluated hexagons
	local openset={start} -- evaluated hexagons
	local current -- where are we at the moment?
	local startTimer=love.timer.getTime()
	local path={} -- path
	while (#openset>0) do
		print("!!!")
		local lowestIndex=1
		for i=1,#openset do
			local opensetD=hexes.getHexagon(openset[i])
			if opensetD.f<hexes.getHexagon(openset[lowestIndex]).f then
				lowestIndex=i
			end
			if opensetD.f==hexes.getHexagon(openset[lowestIndex]).f then
				if (opensetD.g>hexes.getHexagon(openset[lowestIndex]).g) then
					lowestIndex=i
				end
			end
		end
		
		local cur=openset[lowestIndex]
		local curData=hexes.getHexagon(cur)
		for _, v in pairs(closedset) do
			local data=hexes.getHexagon(v)
			data["type"]["color"]={255,0,0} -- for some reason this is the path?? I'll live with it.
		end
		for _, v2 in pairs(openset) do
			local data=hexes.getHexagon(v2)
			data["type"]["color"]={0,255,0} -- for some reason this is the path?? I'll live with it.
		end		

		if cur==goal then
			local temp=cur
			print("prev:",temp.previous)
			local endT=love.timer.getTime()
			print("Done in: "..tostring(round2(endT-startTimer,3)))
			while (temp.previous) do
				local newTemp=temp.previous
				temp=newTemp
				local tempData=hexes.getHexagon(temp)
				tempData["type"]["color"]={255,255,255} -- draw path as white
			end
			break
		end
		
		remove(openset,cur)
		table.insert(closedset,cur)
		
		local neighbors=grid:neighbors(cur.q,cur.r);
		for neighborInd, neighbor in pairs(neighbors) do
			local neighbors=neighbor
			local neighborsD=hexes.getHexagon(neighbor)
			if neighborsD["text"]~="1" then
				if not find(closedset,neighbor) then
					local tempG=curData.g+1
					if find(openset,neighbor) then
						if tempG<neighborsD.g then
							neighborsD.g=tempG
						end
					else
						neighborsD.g=tempG
						table.insert(openset,neighbor)
					end
				end
				neighborsD.h=AStar.heuristic(neighbor,goal)
				neighborsD.f=neighborsD.g+neighborsD.h
				neighborsD.previous=cur
			end
		end		
	end
	return {finished and closedset or nil,finished}
end

return AStar


