local colors = require 'lib/MaterialDesign/colors'

local Menu={}
Menu.__index=Menu
local buttonFont = love.graphics.newFont("Assets/Roboto-Black.ttf",24)
local titleFont = love.graphics.newFont("Assets/Roboto-Bold.ttf",38)
local headerFont = love.graphics.newFont("Assets/Roboto-Medium.ttf",31)
local textFont = love.graphics.newFont("Assets/Roboto-Thin.ttf",16)
function Menu.new()
	local M=setmetatable({},Menu)
	M.Settings={
		["difficulty"]=100
	}
	M.Buttons={
		startButton={name="start",w=200,h=30,x=0,y=200,text="START",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=true,
			func = function(self)
				M.Active=false
			end
		},
		howToPlayButton={name="HowToPlay",w=200,h=30,x=0,y=250,text="HOW TO PLAY",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=true,
			func=function(self)
				for _, b in pairs(self.Buttons) do
					if b.name~="back" then
						b.active=false
					else
						b.active=true
					end
				end
				for _, text in pairs(self.Text) do
					if text.name~="HowToPlayText" then
						text.active=false
					else
						text.active=true
					end
				end
			end
		},
		settingsButton={name="settings",w=200,h=30,x=0,y=300,text="SETTINGS",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=true,
			func=function(self)
				for _, b in pairs(self.Buttons) do
					if b.name=="easy" or b.name=="medium" or b.name=="hard" then
						b.active=true
					else
						b.active=false
					end
				end
				for _, t in pairs(self.Text) do
					if t.name~="settingsHeader" and t.name ~="difficultyHeader" then
						t.active=false
					else
						t.active=true
					end
				end
				self.Buttons.backButton.active=true
			end
		},
		backButton={name="back",w=200,h=30,x=400,y=570,text="BACK",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=false,
			func=function(self)
				local buttons=self.Buttons
				buttons.easyButton.active=false
				buttons.medButton.active=false
				buttons.hardButton.active=false				
				buttons.backButton.active=false
				buttons.startButton.active=true
				buttons.howToPlayButton.active=true
				buttons.settingsButton.active=true
				for _, text in pairs(self.Text) do
					if text.name=="header" then
						text.active=true
					else
						text.active=false
					end
				end
				self.Buttons.backButton.hovering=false
			end
		},
		easyButton={name="easy",w=200,h=30,x=0,y=250,text="EASY",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=false,
			func=function(self)
				self.Settings["difficulty"]=100
			end
		},
		medButton={name="medium",w=200,h=30,x=0,y=300,text="MEDIUM",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=false,
			func=function(self)
				self.Settings["difficulty"]=50
			end
		},
		hardButton={name="hard",w=200,h=30,x=0,y=350,text="HARD",color=colors["black"]["text"],Font=buttonFont,hovering=false,active=false,
			func=function(self)
				self.Settings["difficulty"]=25
			end
		}
	}
	M.Text={
		menuHeader={name="header",w=400,h=30,x=-95,y=168,color={0,0,0},Font=titleFont,active=true,text="MAIN MENU",align='center'},
		howToPlayText={name="HowToPlayText",w=600,h=600,x=0,y=30,color={0,0,0},Font=textFont,active=false,text=[[
HOW TO PLAY:
The goal of this game is to take over all of the enemies hexagons.

You do this by hovering over one of your own hexagons. This will highlight all capturable enemy hexagons that neighbor it.

To capture the highlighted hexagons, you left click on the hexagon you are currently hovering over. This will change the enemies hexagons element to the element of the hexagon that took it over.

Another feature is that you can swap two of your own hexagons given that they are neighboring eachother. To do this, you right click on one of your hexagons and then click on one of the highlighted neighbors
		]],
		align='left'
		},
		settingsHeader={name="settingsHeader",w=200,h=30,x=0,y=168,color={0,0,0},Font=titleFont,active=false,text="SETTINGS",align='center'},
		difficultyHeader={name="difficultyHeader",w=200,h=30,x=0,y=220,color={0,0,0},Font=headerFont,active=false,text="DIFFICULTY",align='center'}
	}
	M.Active=true
	return M
end

function Menu:draw()
	if self.Active then
		for _, text in pairs(self.Text) do
			if text.active then
				love.graphics.setFont(text.Font)
				love.graphics.setColor(text.color)
				love.graphics.printf(text.text,text.x,text.y,text.w,text.align)
			end
		end
		for _, button in pairs(self.Buttons) do
			if button.active then
				love.graphics.setFont(button.Font)
				love.graphics.setColor(button.color)
				love.graphics.printf(button.text,button.x,button.y+3,button.w,'center')
			end
		end
	end
end

function Menu:click(x,y)
	for _, button in pairs(self.Buttons) do
		if button.hovering then
			button.func(self)
		end
	end
end

function Menu:update()
	local mx,my=love.mouse.getPosition()
	local function pointInRectangle(pointx, pointy, rectx, recty, rectwidth, rectheight)
		return pointx > rectx and pointy > recty and pointx < rectx + rectwidth and pointy < recty + rectheight
	end
	for _, button in pairs(self.Buttons) do
		if button.active then
			button.hovering=pointInRectangle(mx,my,button.x,button.y,button.w,button.h)
			button.color=button.hovering and colors["grey"]["500"] or colors["black"]["text"]
		end
	end
end

return Menu